@@ -1,7 +1,5 @@
 package Pixie::Complicity;
 
-our $VERSION = '2.08_02';
-
 $Pixie::Complicity::Loaded++;
 
 =head1 NAME
@@ -117,7 +115,7 @@ Another boolean. Used by Pixie to know whether an object in this class
 should be immediately fetched in cases where Pixie would normally use
 a Pixie::Proxy object to provide deferred loading. You generally want
 to use this for objects that get accessed directly (you naughty
-encapsulation violator you), because a L<Pixie::Proxy> only fetches the
+encapsulation violator you), because a Pixie::Proxy only fetches the
 real thing when it notices a method call to the object.
 
 =item px_as_rawstruct
@@ -157,7 +155,6 @@ init method to set up stuff based on the instance variables...)
 package UNIVERSAL;
 
 use Scalar::Util qw/ blessed weaken reftype isweak /;
-use strict;
 
 sub px_is_storable {
   my $self = shift;
@@ -166,8 +163,8 @@ sub px_is_storable {
 }
 
 sub px_class {
-  my $class = shift;
-  return ref($class) || $class
+  my $proto = shift;
+  return ref($proto) || $proto
 }
 
 sub px_oid { $_[0]->PIXIE::oid };
@@ -175,7 +172,6 @@ sub px_oid { $_[0]->PIXIE::oid };
 sub px_freeze { shift }
 sub px_thaw { shift }
 sub px_is_immediate { }
-sub px_in_rootset { 1 }
 
 sub px_as_rawstruct {
   my $self = shift;
@@ -194,22 +190,62 @@ sub px_as_rawstruct {
 }
 
 sub px_empty_new {
-  my $class = shift;
-  $class->new;
+  my $proto = shift;
+  $proto->new;
 }
 
-sub px_do_final_restoration {
-  my $self = shift;
-  Pixie->get_the_current_pixie->make_new_object($self, ref($self));
-}
-
-
 # 'Internal' methods
 
 
-sub _px_extraction_freeze { Pixie->get_the_current_pixie->extraction_freeze(@_) }
-sub _px_extraction_thaw   { Pixie->get_the_current_pixie->extraction_thaw(@_)   }
-sub _px_insertion_freeze  { Pixie->get_the_current_pixie->insertion_freeze(@_)  }
-sub _px_insertion_thaw    { Pixie->get_the_current_pixie->insertion_thaw(@_)    }
+sub _px_extraction_freeze {
+  shift;
+}
+
+sub _px_extraction_thaw {
+  my $self  = shift;
+  my $class = ref($self);
+  my $pixie = Pixie->get_the_current_pixie;
+  my $oid   = Pixie->get_the_current_oid;
+  $self = $self->px_thaw;
+
+  my $real_obj = $pixie->make_new_object($self, ref($self));
+
+  bless $self, 'Class::Whitehole' unless
+    $self->PIXIE::address == $real_obj->PIXIE::address;
+
+  $pixie->{_objectmanager}->bind_object_to_oid($real_obj => $oid);
+  $real_obj->PIXIE::oid eq $oid or die "Bad OID stuff";
+  $pixie->cache_insert($real_obj);
+  return $real_obj;
+}
+
+sub _px_insertion_freeze {
+  my $self = shift;
+  my $pixie = Pixie->get_the_current_pixie;
+  $pixie->ensure_storability($self);
+  my($oid) = $pixie->cache_insert($self);
+  $self = $self->px_freeze;
+  bless {oid     => $oid,
+	 class   => ref($self),
+	 content => $self->px_as_rawstruct }, Pixie::ObjHolder;
+}
+
+sub _px_insertion_thaw {
+  my $self = shift;
+  Carp::confess "UNIVERSAL::_px_insertion_thaw should never be reached!";
+}
+
+sub Pixie::ObjHolder::_px_insertion_thaw {
+  my $obj_holder = shift;
+  my $class = $obj_holder->{class};
+  my $self = bless $obj_holder->{content}, $class;
+
+  my $pixie = Pixie->get_the_current_pixie;
+  $pixie->{_objectmanager}->bind_object_to_oid($self, $obj_holder->{oid});
+  $pixie->ensure_storability($self);
+  my $retval = $pixie->store_individual_at($self, $obj_holder->{oid});
+  bless $self, 'Class::Whitehole';
+  return $retval;
+}
 
 1;
